@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -36,18 +36,20 @@
 ##    [6,1] = VideoResolution
 ##    [7,1] = VideoFrameInterval
 ##    [8,1] = VideoFormat
-##    [9,1] = brightness
-##    [10,1] = contrast
-##    [11,1] = saturation
-##    [12,1] = white_balance_temperature_auto
-##    [13,1] = gain
-##    [14,1] = power_line_frequency
-##    [15,1] = white_balance_temperature
-##    [16,1] = sharpness
-##    [17,1] = backlight_compensation
-##    [18,1] = exposure_auto
-##    [19,1] = exposure_absolute
-##    [20,1] = exposure_auto_priority
+##    [9,1] = user_controls
+##    [10,1] = brightness
+##    [11,1] = contrast
+##    [12,1] = saturation
+##    [13,1] = white_balance_automatic
+##    [14,1] = gain
+##    [15,1] = power_line_frequency
+##    [16,1] = white_balance_temperature
+##    [17,1] = sharpness
+##    [18,1] = backlight_compensation
+##    [19,1] = camera_controls
+##    [20,1] = auto_exposure
+##    [21,1] = exposure_time_absolute
+##    [22,1] = exposure_dynamic_framerate
 ##  @}
 ## @end group
 ## @end example
@@ -76,27 +78,27 @@ function val = get (vi, prop)
     print_usage();
   endif
   if (nargin == 1)
-    ctrls = __v4l2_handler_queryctrl__(vi.imaqh);
+    ctrls = __imaq_handler_queryctrl__(vi.imaqh);
     val = vertcat(__property_names__(vi), fieldnames(ctrls));
   else
     switch (prop)
       case "SelectedSourceName"
         val = vi.SelectedSourceName;
       case "DeviceCapabilities"
-        val = __v4l2_handler_querycap__(vi.imaqh);
+        val = __imaq_handler_querycap__(vi.imaqh);
       case "VideoInput"
-        val = __v4l2_handler_g_input__(vi.imaqh);
+        val = __imaq_handler_get_input__(vi.imaqh);
       case "VideoFrameInterval"
-        val = __v4l2_handler_g_parm__(vi.imaqh);
+        val = __imaq_handler_get_frameinterval__(vi.imaqh);
       case "VideoResolution"
-        val = __v4l2_handler_g_fmt__(vi.imaqh).size;
+        val = __imaq_handler_g_fmt__(vi.imaqh).size;
       case "VideoFormat"
-        val = __v4l2_handler_g_fmt__(vi.imaqh).pixelformat;
+        val = __imaq_handler_g_fmt__(vi.imaqh).fourcc;
       otherwise
         # get controls
-        ctrls = __v4l2_handler_queryctrl__(vi.imaqh);
+        ctrls = __imaq_handler_queryctrl__(vi.imaqh);
         if (isfield(ctrls, prop))
-          val = __v4l2_handler_g_ctrl__(vi.imaqh, ctrls.(prop).id);
+          val = __imaq_handler_g_ctrl__(vi.imaqh, ctrls.(prop).id);
         else
           error ("videoinput: get: invalid property name '%s'", prop);
         endif
@@ -106,19 +108,19 @@ function val = get (vi, prop)
 endfunction
 
 %!test
-%! obj = videoinput("v4l2", __test__device__);
+%! obj = videoinput(__test__device__{:});
 %! props = get(obj);
-%! assert(get(obj, "SelectedSourceName"), __test__device__);
+%! assert(get(obj, "SelectedSourceName"), __test__device__{2});
 %! caps = get(obj, "DeviceCapabilities");
 %! video_in = get(obj, "VideoInput");
 %! s = get(obj, "VideoResolution");
 
 %!test
-%! obj = videoinput("v4l2", __test__device__);
+%! obj = videoinput(__test__device__{:});
 %! T = get(obj, "VideoFrameInterval");
 
 %!test
-%! obj = videoinput("v4l2", __test__device__);
+%! obj = videoinput(__test__device__{:});
 %! f = get(obj, "VideoFormat");
 
-%!error get(obj, "there_is_no_such_property")
+%!error <invalid property name> get(videoinput(__test__device__{:}), "there_is_no_such_property")
