@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -14,15 +14,20 @@
 ## this program; if not, see <http:##www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {[@var{img}, @var{seq}, @var{ts}, @var{tc}] =} getsnapshot (@var{vi}, [@var{preview}])
-## Get a snapshot from a videoinput object buffer.
+## @deftypefn {Function File} {[@var{img}, @var{seq}, @var{ts}, @var{tc}] =} getsnapshot (@var{vi}, [@var{preview}], [@var{raw}])
+## Get a snapshot from a videoinput object.
 ## Streaming has to be enabled before calling getsnapshot.
-## If @var{preview}==true the captured image is also shown in a separate FLTK window.
+## If @var{preview} == true the captured image is also shown in a separate window.
 ##
 ## @table @var
 ## @item img
-## Captured image. The type and size of @var{img} depends on the @qcode{VideoFormat} property of @var{vi}.
-## H and W below refers to the height and width returned from @code{get(VI, "VideoResolution")}.
+## Captured image.
+##
+## If @var{raw} == false (default) the format returned from the driver is
+## converted to RGB3. Width and height matches @code{get(VI, "VideoResolution")}
+##
+## If @var{raw} == true the format returned by the driver
+## (see @qcode{VideoFormat} property) is returned.
 ##
 ##   @table @var
 ##   @item @qcode{RGB3}, @qcode{RGB24}
@@ -41,7 +46,7 @@
 ##   frame to frame due to compression. You can save this as JPEG (add a huffman table) with
 ##   @example
 ##   @group
-##     obj = videoinput("v4l2", "/dev/video0");
+##     obj = videoinput();
 ##     set (obj, "VideoFormat", "MJPG");
 ##     start (obj);
 ##     img = getsnapshot (obj);
@@ -64,78 +69,73 @@
 ##   @end table
 ##
 ## @item struct @var{tc}
-## Timecode, see @url{http://linuxtv.org/downloads/v4l-dvb-apis/buffer.html#v4l2-timecode}
+## Timecode, see @url{https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/buffer.html#c.V4L.v4l2_timecode}
 ## @end table
 ## @seealso {@@videoinput/start, @@videoinput/preview}
 ## @end deftypefn
 
-function [img, seq, timestamp, timecode] = getsnapshot (vi, pv=0)
+function [img, seq, timestamp, timecode] = getsnapshot (vi, preview = 0, raw = 0)
 
-  if (nargin < 1 || nargin>2)
+  if (nargin < 1 || nargin > 3)
     print_usage();
   endif
   if (nargout <= 3)
-    [img, seq, timestamp] = __v4l2_handler_capture__(vi.imaqh, pv);
+    [img, seq, timestamp] = __imaq_handler_capture__(vi.imaqh, preview, raw);
   else
-    [img, seq, timestamp, timecode] = __v4l2_handler_capture__(vi.imaqh, pv);
+    [img, seq, timestamp, timecode] = __imaq_handler_capture__(vi.imaqh, preview, raw);
   endif
-  #fmt = __v4l2_handler_g_fmt__(vi.imaqh).pixelformat;
+  #fmt = __imaq_handler_g_fmt__(vi.imaqh).pixelformat;
   #printf ("pixelformat = -%s-\n", fmt);
 
 endfunction
 
 %!test
-%! obj = videoinput("v4l2", __test__device__);
+%! obj = videoinput(__test__device__{:});
 %! oldval = get(obj, "VideoResolution");
 %! default_size = set (obj, "VideoResolution")(1,:);
 %! set (obj, "VideoResolution", default_size);
-%! set (obj, 'VideoFormat', 'RGB24');
+%! # We expect here, that every driver (v4l2 and Media Foundation)
+%! # on earth would support YUYV. This might not be the case...
+%! set (obj, 'VideoFormat', 'YUYV');
 %! start (obj, 2)
 %! img = getsnapshot (obj);
-%! img = getsnapshot (obj, 1);
-%! [img, seq] = getsnapshot (obj, 1);
+%! do_preview = !isempty(getenv("DISPLAY")); # only if there is a display
+%! img = getsnapshot (obj, do_preview);
+%! [img, seq] = getsnapshot (obj, do_preview);
 %! # The v4l2 loopback device doesn't support the seqence numbering and returns always 0
 %! if (!strcmp(get(obj, "DeviceCapabilities").driver, "v4l2 loopback"))
 %!   assert (seq >= 2);
 %! endif
 %! [img, seq, T] = getsnapshot(obj, 0);
-%! assert (isstruct(T))
 %! stop (obj)
 %! set (obj, "VideoResolution", oldval);
 
 %!test
-%! obj = videoinput("v4l2", __test__device__);
-%! set (obj, "VideoFormat", "RGB3")
-%! start (obj)
-%! img = getsnapshot (obj);
-%! stop (obj)
-
-%!test
-%! obj = videoinput ("v4l2", __test__device__);
-%! # see http://www.linuxtv.org/downloads/v4l-dvb-apis/V4L2-PIX-FMT-YUYV.html
-%! set (obj, "VideoFormat", "YUYV")
-%! start (obj)
-%! img = getsnapshot (obj);
-%! stop (obj)
-
-%!test
-%! obj = videoinput ("v4l2", __test__device__);
-%! # see see http://www.linuxtv.org/downloads/v4l-dvb-apis/re23.html
-%! set (obj, "VideoFormat", "YU12")
-%! start (obj)
-%! img = getsnapshot(obj);
-%! stop (obj)
+%! obj = videoinput (__test__device__{:});
+%! fmts = {set(obj,"VideoFormat").fourcc};
+%! for k = 1:numel (fmts)
+%!   set (obj, "VideoFormat", fmts{k});
+%!   s = get (obj, "VideoResolution");
+%!   start (obj)
+%!   # try to internally convert to RGB3
+%!   # this might fail if there is no converter (yet)
+%!   try
+%!     img = getsnapshot (obj);
+%!     assert (size (img), [fliplr(s) 3]);
+%!   catch ME
+%!     if (! strcmp (ME.identifier, 'image-acquisition:getsnapshot:unsupported-video-format'))
+%!       stop (obj);
+%!       rethrow (ME);
+%!     endif
+%!   end_try_catch
+%!   # getting a raw representation (like for H264) should always work
+%!   img = getsnapshot (obj, false, true); # ret raw
+%!   stop (obj)
+%! endfor
 
 %!demo
-%! obj = videoinput ("v4l2", __test__device__);
-%! # see http://www.linuxtv.org/downloads/v4l-dvb-apis/V4L2-PIX-FMT-YUYV.html
-%! set(obj,"VideoFormat","YUYV")
-%! start(obj)
-%! img = getsnapshot(obj);
-%! tmp = cat (3, img.Y, kron(img.Cb, [1 1]), kron(img.Cr, [1 1]));
-%! # convert to RGB with octave-forge image function ycbcr2rgb
-%! pkg load image
-%! rgb = ycbcr2rgb (tmp, "709");
-%! image(rgb)
-%! title ("YUYV, Standard 709")
-%! stop(obj)
+%! obj = videoinput ();
+%! start (obj)
+%! img = getsnapshot (obj);
+%! image (img)
+%! stop (obj)
