@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -24,15 +24,13 @@ function preview (vi)
     print_usage();
   endif
   unwind_protect
-    __v4l2_handler_streamon__(vi.imaqh, 2);
-    disp("Hit CTRL+C to exit")
+    __imaq_handler_streamon__(vi.imaqh, 2);
+    disp("Hit CTRL+C or close preview window to exit")
     fflush(stdout);
-    # Show preview window if it's not already shown
-    __v4l2_handler_capture__(vi.imaqh, 1);
-    while(__v4l2_preview_window_is_shown__(vi.imaqh))
-      __v4l2_handler_capture__(vi.imaqh, 2);
-    endwhile
+    do
+      __imaq_handler_capture__(vi.imaqh, 1, 0);
+    until(!__imaq_preview_window_is_shown__(vi.imaqh))
   unwind_protect_cleanup
-    __v4l2_handler_streamoff__(vi.imaqh);
+    __imaq_handler_streamoff__(vi.imaqh);
   end_unwind_protect
 endfunction
