@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -79,37 +79,37 @@ function ret = set (vi, varargin)
             error ("Use set (VI, 'VideoFormat', FMT) to specify the returned image format");
           case 'VideoResolution'
             if (isvector (val) && isreal (val) && length (val) == 2)
-              __v4l2_handler_s_fmt__(vi.imaqh, "", val);
+              current_fmt = get(vi, "VideoFormat");
+              __imaq_handler_s_fmt__(vi.imaqh, current_fmt, val);
             else
               error ('set VideoResolution: expects a real vector [width height]');
             endif
           case 'VideoInput'
             if (isscalar (val) && isreal (val))
-              __v4l2_handler_s_input__(vi.imaqh, val);
+              __imaq_handler_set_input__(vi.imaqh, val);
             else
               error ('set VideoInput: expecting the value to be a scalar integer');
             endif
           case 'VideoFrameInterval'
             if (ismatrix (val) && isreal (val) && numel (val) == 2)
-              __v4l2_handler_s_parm__(vi.imaqh, val);
+              __imaq_handler_set_frameinterval__(vi.imaqh, val);
             else
               error ('set VideoFrameInterval: expecting a 1x2 matrix with [numerator, denominator]');
             endif
           case 'VideoFormat'
             if (ischar (val))
-              __v4l2_handler_s_fmt__(vi.imaqh, val, [0 0]);
+              __imaq_handler_s_fmt__(vi.imaqh, val, [0 0]);
             else
               error ('set VideoFormat: expecting a string');
             endif
           otherwise
             if (!__is_read_only_property__(prop))
-              # could be a v4l2 control
-              ctrls = __v4l2_handler_queryctrl__(vi.imaqh);
+              ctrls = __imaq_handler_queryctrl__(vi.imaqh);
               if (isfield(ctrls, prop))
-                __v4l2_handler_s_ctrl__(vi.imaqh, ctrls.(prop).id, val);
-                v = __v4l2_handler_g_ctrl__(vi.imaqh, ctrls.(prop).id);
+                __imaq_handler_s_ctrl__(vi.imaqh, ctrls.(prop).id, val);
+                v = __imaq_handler_g_ctrl__(vi.imaqh, ctrls.(prop).id);
                 if ( val != v)
-                  warning("v4l2 driver limited set value %d to %d", val, v);
+                  warning("driver limited set value %d to %d", val, v);
                 endif
               else
                 error ('set: invalid property of videoinput class');
@@ -132,20 +132,20 @@ function ret = __list_range__ (vi, prop)
           error ("Use set (VI, 'VideoFormat') to get a list of supported formats.")
         case 'VideoInput'
           # enumerate available inputs
-          ret = __v4l2_handler_enuminput__ (vi.imaqh);
+          ret = __imaq_handler_enum_inputs__ (vi.imaqh);
         case 'VideoResolution'
           # enumerate possible framerates
-          fmt = __v4l2_handler_g_fmt__(vi.imaqh).pixelformat;
-          ret = __v4l2_handler_enum_framesizes__ (vi.imaqh, fmt);
+          fmt = __imaq_handler_g_fmt__(vi.imaqh).fourcc;
+          ret = __imaq_handler_enum_framesizes__ (vi.imaqh, fmt);
         case 'VideoFrameInterval'
           # return possible frameintervals for currently selected framesize
-          fmt = __v4l2_handler_g_fmt__(vi.imaqh).pixelformat;
-          current_frame_size = __v4l2_handler_g_fmt__ (vi.imaqh).size;
-          ret = __v4l2_handler_enum_frameintervals__ (vi.imaqh, current_frame_size, fmt);
+          fmt = __imaq_handler_g_fmt__(vi.imaqh).fourcc;
+          current_frame_size = __imaq_handler_g_fmt__ (vi.imaqh).size;
+          ret = __imaq_handler_enum_frameintervals__ (vi.imaqh, fmt, current_frame_size);
         case 'VideoFormat'
-          ret = __v4l2_handler_enum_fmt__ (vi.imaqh);
+          ret = __imaq_handler_enum_formats__(vi.imaqh);
         otherwise ## perhaps a v4l2 control?
-          ctrls = __v4l2_handler_queryctrl__(vi.imaqh);
+          ctrls = __imaq_handler_queryctrl__(vi.imaqh);
           if (isfield(ctrls, prop))
             # yes, it is a v4l2 property
             ret = getfield(ctrls, prop);
@@ -171,7 +171,7 @@ function ret = __is_read_only_property__ (prop)
 endfunction
 
 %!test
-%! obj = videoinput ("v4l2", __test__device__);
+%! obj = videoinput (__test__device__{:});
 %! svi = set (obj, "VideoInput");
 %! assert (isstruct(svi))
 %! set(obj, "VideoInput", 0);
@@ -186,26 +186,23 @@ endfunction
 %!error set(obj, "DeviceCapabilities")
 
 %!test
-%! obj = videoinput ("v4l2", __test__device__);
-%! set (obj, 'VideoFormat', 'RGB24');
-
-%!test
-%! obj = videoinput ("v4l2", __test__device__);
+%! obj = videoinput (__test__device__{:});
 %! fmts = set (obj, 'VideoFormat');
-%! set (obj, 'VideoFormat', fmts(end).pixelformat);
-%! set (obj, 'VideoFormat', 'RGB24');
+%! set (obj, 'VideoFormat', fmts(1).fourcc);
+%! set (obj, 'VideoFormat', fmts(end).fourcc);
 
 %!test
-%! obj = videoinput ("v4l2", __test__device__);
+%! obj = videoinput (__test__device__{:});
 %! T = set (obj, 'VideoFrameInterval');
-%! # not all drives support enumeration and query of trameintervals
+%! # not all drives support enumeration and query of frameintervals
 %! if (rows(T) >= 1)
 %!   set (obj, 'VideoFrameInterval', T(1,:));
 %!   set (obj, 'VideoFrameInterval', T(end,:));
 %! endif
 
-%!warning
-%! obj = videoinput ("v4l2", __test__device__);
-%! # This shouldn't be supported by any camera and the driver
-%! # clamps this to valid values but a warning should be displayed
-%! set (obj, 'VideoFrameInterval', [1 10000]);
+# FIXME: this only works for v4l2
+#%!warning
+#%! obj = videoinput (__test__device__{:});
+#%! # This shouldn't be supported by any camera and the driver
+#%! # clamps this to valid values but a warning should be displayed
+#%! set (obj, 'VideoFrameInterval', [1 10000]);
