@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -23,7 +23,7 @@ function start (vi, n = 2)
   if (nargin > 2 || nargin < 1)
     print_usage();
   endif
-  __v4l2_handler_streamon__(vi.imaqh, n);
+  __imaq_handler_streamon__(vi.imaqh, n);
 endfunction
 
 # already tested in getsnapshot
