@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2024-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -14,14 +14,14 @@
 ## this program; if not, see <http:##www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {@var{vi} =} videoinput (@var{adaptorname}, @var{device})
-## @deftypefnx {Function File} {@var{vi} =} videoinput (@var{adaptorname}, @var{device}, @var{format})
-## Initializes a new video input object. Currently only v4l2 is available as adaptorname.
+## @deftypefn {Function File} {@var{vi} =} videoinput ([@var{adaptorname}, @var{device}, [@var{format}]])
+## Initializes a new video input object. Currently only "v4l2" and "mf" is available as adaptorname.
+## If called without arguments, the first available device is used.
 ##
 ## @group
 ## @example
 ## obj = videoinput("v4l2", "/dev/video0")
-## @result{} obj = videoinput for v4l2
+## @result{} obj = videoinput
 ##      device             = /dev/video0
 ##      driver             = uvcvideo
 ##      card               = UVC Camera (046d:0825)
@@ -30,36 +30,40 @@
 ##      VideoFormat = MJPEG
 ## @end example
 ## @end group
-## @seealso{@@videoinput/getsnapshot}
+## @seealso{imaqhwinfo, @@videoinput/getsnapshot}
 ## @end deftypefn
 
 function vi = videoinput (adaptorname, device, format)
-  # defaults
-  vidata = struct ("SelectedSourceName", "/dev/video0");
 
-  if (nargin == 0 || nargin==1 || nargin>3)
+  if (nargin == 0)
+    l = imaqhwinfo ();
+    n = numel (l);
+    if (n == 0)
+      error("It appears that you do not have have a compatible video capture device installed.");
+    else
+      # silently ignore if there is more than one device
+      if (ispc ())
+        adaptorname = "mf";
+        vidata.PrettyName = l(1).name;
+        device = l(1).symlink;
+      elseif (isunix ())
+        adaptorname = "v4l2";
+        device = l(1).device;
+      endif
+    endif
+  elseif (nargin == 1 || nargin > 3 || (nargin >= 2 && ! ischar(device)))
     print_usage();
   endif
-  if (nargin == 2)
-    if (strcmp(adaptorname, "v4l2"))
-      if (ischar(device))
-        vidata.SelectedSourceName = device;
-      else
-        print_usage();
-      endif
-    else
-      error("Only v4l2 adaptors are possible yet")
-    endif
-  endif
 
-  vidata.imaqh = __v4l2_handler_open__(vidata.SelectedSourceName);
+  vidata.SelectedSourceName = device;
+  vidata.imaqh = __imaq_handler_open__(adaptorname, device);
   vi = class (vidata, "videoinput");
 
   if (nargin == 3)
-    set(vi, "VideoResolution", format);
+    set (vi, "VideoResolution", format);
   endif
 
 endfunction
 
 %!test
-%! obj = videoinput("v4l2", __test__device__);
+%! obj = videoinput(__test__device__(){:});
