@@ -1,4 +1,4 @@
-## Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+## Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify it under
 ## the terms of the GNU General Public License as published by the Free Software
@@ -33,8 +33,9 @@
 ## @end deftypefn
 
 function ret = imaqhwinfo()
-  ret = __v4l2_list_devices__ ();
+  ret = __imaq_enum_devices__ ();
 endfunction
 
 %!test
 %! d = imaqhwinfo();
+%! assert (numel (d) >= 1);
