@@ -1,4 +1,4 @@
-// Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+// Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU General Public License as published by the Free Software
@@ -13,9 +13,15 @@
 // You should have received a copy of the GNU General Public License along with
 // this program; if not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
+#ifdef HAVE_LIBV4L2_H
+
+#include <cassert>
+#include <dirent.h>
+//#include <sys/types.h>
 #include "cl_v4l2_handler.h"
 
-#define ARRAY_SIZE(a)	(sizeof(a)/sizeof((a)[0]))
+#define ARRAY_SIZE(a) (sizeof(a)/sizeof((a)[0]))
 
 static std::string
 num2s (unsigned num) //taken from v4l2-ctl.cpp
@@ -57,10 +63,12 @@ buftype2s (int type) //taken from v4l2-ctl.cpp
     }
 }
 
-static struct { //taken from yavta Copyright (C) 2005-2010 Laurent Pinchart
+static struct   //taken from yavta Copyright (C) 2005-2010 Laurent Pinchart
+{
   const char *name;
   unsigned int fourcc;
-} pixel_formats[] = {
+} pixel_formats[] =
+{
   { "RGB332", V4L2_PIX_FMT_RGB332 },
   { "RGB555", V4L2_PIX_FMT_RGB555 },
   { "RGB565", V4L2_PIX_FMT_RGB565 },
@@ -108,60 +116,59 @@ static struct { //taken from yavta Copyright (C) 2005-2010 Laurent Pinchart
 static std::string v4l2_fourcc_name(unsigned int fourcc)
 {
   static char name[5];
-  for (int i = 0; i < 4; ++i) {
-    name[i] = fourcc & 0xff;
-    fourcc >>= 8;
-  }
+  for (int i = 0; i < 4; ++i)
+    {
+      name[i] = fourcc & 0xff;
+      fourcc >>= 8;
+    }
   name[4] = '\0';
-  return string(name);
+  return std::string(name);
 }
 
 static std::string v4l2_format_name(unsigned int fourcc)
 {
-  for (unsigned int i = 0; i < ARRAY_SIZE(pixel_formats); ++i) {
-    if (pixel_formats[i].fourcc == fourcc)
-      return string(pixel_formats[i].name);
-  }
+  for (unsigned int i = 0; i < ARRAY_SIZE(pixel_formats); ++i)
+    {
+      if (pixel_formats[i].fourcc == fourcc)
+        return std::string(pixel_formats[i].name);
+    }
   return v4l2_fourcc_name(fourcc);
 }
 
 static unsigned int v4l2_format_code(const char *name)
 {
-  unsigned int i;
-
-  for (i = 0; i < ARRAY_SIZE(pixel_formats); ++i) {
-    if (strcasecmp(pixel_formats[i].name, name) == 0)
-      return pixel_formats[i].fourcc;
-  }
+  for (unsigned int i = 0; i < ARRAY_SIZE(pixel_formats); ++i)
+    {
+      if (strcasecmp(pixel_formats[i].name, name) == 0)
+        return pixel_formats[i].fourcc;
+    }
 
   //try fourcc format
   unsigned int fourcc = 0;
-  for (int i = 3; i >=0; i--) {
-    fourcc <<= 8;
-    fourcc += name[i];
-  }
+  for (int i = 3; i >=0; i--)
+    {
+      fourcc <<= 8;
+      fourcc += name[i];
+    }
   return fourcc;
 }
 
-DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(v4l2_handler, "v4l2_handler", "v4l2_handler");
+//DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(v4l2_handler, "v4l2_handler", "v4l2_handler");
 
-bool v4l2_handler::type_loaded = false;
+//bool v4l2_handler::type_loaded = false;
 
 v4l2_handler::v4l2_handler ()
-  : octave_base_value(),
+  : imaq_handler(),
     fd(-1), n_buffer(0), buffers(0), streaming(0),
-    preview_window(0), _is_video_capture (0), _is_meta_capture(0)
+    _is_video_capture (0), _is_meta_capture(0)
 {
-  //octave_stdout << "v4l2_handler C'Tor, type_loaded = " << type_loaded << endl;
-  if (!type_loaded)
-    {
-      type_loaded = true;
-      v4l2_handler::register_type();
-    }
+  //octave_stdout << "v4l2_handler C'Tor, type_id() = " << type_id() << std::endl;
 }
 
 v4l2_handler::v4l2_handler (const v4l2_handler& m)
-  : octave_base_value()
+  : imaq_handler(),
+    fd(-1), n_buffer(0), buffers(0), streaming(0),
+    _is_video_capture (0), _is_meta_capture(0)
 {
   octave_stdout << "v4l2_handler: the copy constructor shouldn't be called" << std::endl;
 }
@@ -169,23 +176,70 @@ v4l2_handler::v4l2_handler (const v4l2_handler& m)
 v4l2_handler::~v4l2_handler ()
 {
   //octave_stdout << "v4l2_handler D'Tor " << endl;
-
-  // delete preview_window if active
-  if (preview_window)
-    {
-      delete preview_window;
-      preview_window = 0;
-    }
-
   // stop streaming, unmap & free buffers, close v4l2 device
   close();
 }
 
 void
-v4l2_handler::print (std::ostream& os, bool pr_as_read_syntax = false) const
+v4l2_handler::print (std::ostream& os, bool pr_as_read_syntax = false)
 {
-  os << "This is class v4l2_handler" << endl;
-  os << "dev = " << dev << ", fd = " << fd << ", n_buffer = " << n_buffer << ", streaming = " << ((streaming)? "true":"false") << endl;
+  os << "This is class v4l2_handler" << std::endl;
+  os << "dev = " << dev << ", fd = " << fd << ", n_buffer = " << n_buffer << ", streaming = " << ((streaming)? "true":"false") << std::endl;
+}
+
+typedef std::vector<std::string> dev_vec;
+
+static bool is_v4l_dev(const char *name)
+{
+  return !memcmp(name, "video", 5) ||
+         !memcmp(name, "radio", 5) ||
+         !memcmp(name, "vbi", 3) ||
+         !memcmp(name, "v4l-subdev", 10);
+}
+
+octave_map
+v4l2_handler::enum_devices ()
+{
+  // Most of this code was taken from v4l2-ctl-common.cpp:list_devices()
+  // which is part of the v4l-utils (http://git.linuxtv.org/v4l-utils.git).
+  // Thanks to Kevin Thayer (Copyright (C) 2003-2004),
+  // Hans Verkuil (Copyright (C) 2004, 2006, 2007) and the linuxtv community.
+
+  //octave_stdout << "Use '$ v4l2-ctl --list-devices' for more details." << std::endl;
+
+  octave_map retval;
+  DIR *dp;
+  struct dirent *ep;
+  dev_vec files;
+  dp = opendir("/dev");
+  if (dp == NULL)
+    {
+      error ("Couldn't open /dev/ directory");
+      return octave_map();
+    }
+  while ((ep = readdir(dp)))
+    if (is_v4l_dev(ep->d_name))
+      files.push_back(std::string("/dev/") + ep->d_name);
+  closedir(dp);
+
+  octave_idx_type i=0;
+  for (dev_vec::iterator iter = files.begin();
+       iter != files.end(); ++iter)
+    {
+      //printf ("trying '%s'...\n", iter->c_str());
+      //fflush(stdout);
+      v4l2_handler h;
+      octave_scalar_map caps = h.open(iter->c_str(), true);
+
+      if (! h.is_meta_capture ())
+        {
+          caps.assign ("device", *iter);
+          retval.assign(i++, caps);
+        }
+      //else
+      //printf ("INFO: list device ignores metadata interface '%s'...\n", iter->c_str());
+    }
+  return retval;
 }
 
 // calls to xioctl should never fail.
@@ -207,8 +261,10 @@ v4l2_handler::xioctl_name (int fh, unsigned long int request, void *arg, const c
 }
 
 octave_scalar_map
-v4l2_handler::open (string d, bool quiet)
+v4l2_handler::open (std::string d, bool quiet)
 {
+  //octave_stdout << "v4l2_handler::open d = " << d << " called" << std::endl;
+
   octave_scalar_map ret;
   fd = v4l2_open(d.c_str(), O_RDWR | O_NONBLOCK, 0);
   if (fd < 0)
@@ -218,10 +274,10 @@ v4l2_handler::open (string d, bool quiet)
   else
     {
       dev = d; // store device path for later info output
-	  ret = querycap ().scalar_map_value ();
+      ret = querycap ().scalar_map_value ();
 
-	  if (!quiet && is_meta_capture ())
-	    warning ("Device '%s' is a metadata interface device", d.c_str());
+      if (!quiet && is_meta_capture ())
+        warning ("Device '%s' is a metadata interface device", d.c_str());
     }
   return ret;
 }
@@ -236,7 +292,7 @@ v4l2_handler::expand_cap (unsigned int cap)
 {
   octave_scalar_map ret;
   ret.assign ("raw", cap);
-  #define CHECK_DEVICE_CAPABILITIES_FIELD(X) ret.assign (#X, (bool)(cap & X))
+#define CHECK_DEVICE_CAPABILITIES_FIELD(X) ret.assign (#X, (bool)(cap & X))
 
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_CAPTURE);
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_VIDEO_OUTPUT);
@@ -265,7 +321,9 @@ v4l2_handler::expand_cap (unsigned int cap)
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_STREAMING);
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_META_OUTPUT);
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_TOUCH);
+#ifdef V4L2_CAP_IO_MC
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_IO_MC);
+#endif
   CHECK_DEVICE_CAPABILITIES_FIELD (V4L2_CAP_DEVICE_CAPS);
 
   return ret;
@@ -298,47 +356,27 @@ v4l2_handler::querycap ()
   // The driver fills the device_caps field. This capability can only
   // appear in the capabilities field and never in the device_caps field.
   if (cap.capabilities & V4L2_CAP_DEVICE_CAPS)
-  {
-    st.assign ("device_caps", expand_cap (cap.device_caps));
-    _is_video_capture = cap.device_caps & V4L2_CAP_VIDEO_CAPTURE;
-    _is_meta_capture  = cap.device_caps & V4L2_CAP_META_CAPTURE;
-  }
+    {
+      st.assign ("device_caps", expand_cap (cap.device_caps));
+      _is_video_capture = cap.device_caps & V4L2_CAP_VIDEO_CAPTURE;
+      _is_meta_capture  = cap.device_caps & V4L2_CAP_META_CAPTURE;
+    }
   else
-  {
-    _is_video_capture = cap.capabilities & V4L2_CAP_VIDEO_CAPTURE;
-    _is_meta_capture  = cap.capabilities & V4L2_CAP_META_CAPTURE;
-  }
+    {
+      _is_video_capture = cap.capabilities & V4L2_CAP_VIDEO_CAPTURE;
+      _is_meta_capture  = cap.capabilities & V4L2_CAP_META_CAPTURE;
+    }
 
   return octave_value (st);
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-g-input.html
- */
-int
-v4l2_handler::g_input ()
-{
-  int index;
-  xioctl (fd, VIDIOC_G_INPUT, &index);
-  return index;
-}
-
-/*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-g-input.html
- */
-void
-v4l2_handler::s_input (int index)
-{
-  xioctl (fd, VIDIOC_S_INPUT, &index);
-}
-
-/*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-enuminput.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-enuminput.html
  * see also output from "v4l2-ctl -n"
  * \return octave_map with the enumeration of all inputs
  */
 octave_value
-v4l2_handler::enuminput ()
+v4l2_handler::enum_inputs ()
 {
   octave_map ret;
   struct v4l2_input inp;
@@ -372,18 +410,38 @@ v4l2_handler::enuminput ()
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-enum-fmt.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-g-input.html
+ */
+int
+v4l2_handler::get_input ()
+{
+  int index;
+  xioctl (fd, VIDIOC_G_INPUT, &index);
+  return index;
+}
+
+/*!
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-g-input.html
+ */
+void
+v4l2_handler::set_input (int index)
+{
+  xioctl (fd, VIDIOC_S_INPUT, &index);
+}
+
+/*!
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-enum-fmt.html
  * see also "v4l2-ctl -w --list-formats"
  * \return octave_map with available video formats
  */
 octave_value
-v4l2_handler::enum_fmt (enum v4l2_buf_type type)
+v4l2_handler::enum_formats ()
 {
   octave_map ret;
   struct v4l2_fmtdesc fmt;
   CLEAR(fmt);
   fmt.index = 0;
-  fmt.type = type;
+  fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
   while (v4l2_ioctl (fd, VIDIOC_ENUM_FMT, &fmt) >= 0)
     {
       octave_scalar_map sm;
@@ -401,17 +459,17 @@ v4l2_handler::enum_fmt (enum v4l2_buf_type type)
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-enum-framesizes.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-enum-framesizes.html
  * see also v4l2-ctl --list-formats-ext
  * \param pixel_format e.g. 'RGB24'
  * \return Nx2 Matrix with width, height
  * \sa enum_frameintervals
  */
 Matrix
-v4l2_handler::enum_framesizes (string pixelformat)
+v4l2_handler::enum_framesizes (std::string pixelformat)
 {
   Matrix ret;
-  __u32 pfcode = v4l2_format_code(pixelformat.c_str());
+  uint32_t pfcode = v4l2_format_code(pixelformat.c_str());
   struct v4l2_frmsizeenum frmsize;
   CLEAR(frmsize);
   frmsize.pixel_format = pfcode;
@@ -432,7 +490,7 @@ v4l2_handler::enum_framesizes (string pixelformat)
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-enum-frameintervals.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-enum-frameintervals.html
  * see also v4l2-ctl --list-formats-ext
  * \param pixel_format e.g. 'RGB24'
  * \param width in px
@@ -441,10 +499,10 @@ v4l2_handler::enum_framesizes (string pixelformat)
  * \sa enum_framesizes
  */
 Matrix
-v4l2_handler::enum_frameintervals (string pixelformat, __u32 width, __u32 height)
+v4l2_handler::enum_frameintervals (std::string pixelformat, uint32_t width, uint32_t height)
 {
   Matrix ret;
-  __u32 pfcode = v4l2_format_code(pixelformat.c_str());
+  uint32_t pfcode = v4l2_format_code(pixelformat.c_str());
   struct v4l2_frmivalenum frmival;
   CLEAR(frmival);
   frmival.pixel_format = pfcode;
@@ -469,10 +527,10 @@ v4l2_handler::enum_frameintervals (string pixelformat, __u32 width, __u32 height
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-g-parm.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-g-parm.html
  */
 Matrix
-v4l2_handler::g_parm ()
+v4l2_handler::get_frameinterval ()
 {
   Matrix ret(1,2);
   struct v4l2_streamparm sparam;
@@ -487,17 +545,17 @@ v4l2_handler::g_parm ()
     }
   else
     {
-      warning("v4l2_handler::g_parm: V4L2_CAP_TIMEPERFRAME is not supported");
+      warning("v4l2_handler::get_frameinterval: V4L2_CAP_TIMEPERFRAME is not supported");
       return Matrix(0,0);
     }
   return ret;
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-g-parm.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-g-parm.html
  */
 void
-v4l2_handler::s_parm (Matrix timeperframe)
+v4l2_handler::set_frameinterval (Matrix timeperframe)
 {
   struct v4l2_streamparm sparam;
   CLEAR(sparam);
@@ -510,15 +568,15 @@ v4l2_handler::s_parm (Matrix timeperframe)
       xioctl(fd, VIDIOC_S_PARM, &sparam);
       struct v4l2_fract *tf = &sparam.parm.capture.timeperframe;
       if (!tf->denominator || !tf->numerator)
-        error("v4l2_handler::s_parm: Invalid framerate");
+        error("v4l2_handler::set_frameinterval: Invalid framerate");
 
-      if (tf->numerator != __u32(timeperframe(0)) || tf->denominator != __u32(timeperframe(1)))
-        warning("v4l2_handler::s_parm: driver is using %d/%d as timeperframe but %d/%d was requested",
-                tf->numerator, tf->denominator, __u32(timeperframe(0)), __u32(timeperframe(1)));
+      if (tf->numerator != uint32_t(timeperframe(0)) || tf->denominator != uint32_t(timeperframe(1)))
+        warning("v4l2_handler::set_frameinterval: driver is using %d/%d as timeperframe but %d/%d was requested",
+                tf->numerator, tf->denominator, uint32_t(timeperframe(0)), uint32_t(timeperframe(1)));
     }
   else
     {
-       warning("v4l2_handler::s_parm: V4L2_CAP_TIMEPERFRAME is not supported");
+      warning("v4l2_handler::set_frameinterval: V4L2_CAP_TIMEPERFRAME is not supported");
     }
 }
 
@@ -543,7 +601,7 @@ v4l2_handler::get_osm (struct v4l2_queryctrl queryctrl)
     {
       struct v4l2_querymenu querymenu;
       CLEAR(querymenu);
-      stringstream menu_str;
+      std::stringstream menu_str;
 
       querymenu.id = queryctrl.id;
 
@@ -562,8 +620,8 @@ v4l2_handler::get_osm (struct v4l2_queryctrl queryctrl)
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-queryctrl.html
- * or better http://www.linuxtv.org/downloads/v4l-dvb-apis/extended-controls.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/vidioc-queryctrl.html
+ * or better https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/extended-controls.html
  * because most "Exposure" ctrls are "extended controls"
  *
  * see also v4l2-ctl -L
@@ -606,8 +664,8 @@ v4l2_handler::queryctrl ()
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/control.html
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-g-ctrl.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/control.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l//vidioc-g-ctrl.html
  */
 int
 v4l2_handler::g_ctrl (int id)
@@ -620,17 +678,22 @@ v4l2_handler::g_ctrl (int id)
 }
 
 /*!
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/control.html
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/vidioc-g-ctrl.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media//control.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media//vidioc-g-ctrl.html
  */
 void
-v4l2_handler::s_ctrl (int id, int value)
+v4l2_handler::s_ctrl (int id, octave_value val)
 {
   struct v4l2_control control;
   CLEAR(control);
   control.id = id;
-  control.value = value;
-  xioctl(fd, VIDIOC_S_CTRL, &control);
+  if (val.isempty())
+    error ("v4l2_handler::s_ctrl enable auto not yet implemented");
+  else
+  {
+    control.value = val.int_value ();
+    xioctl(fd, VIDIOC_S_CTRL, &control);
+  }
 }
 
 /*!
@@ -641,7 +704,7 @@ v4l2_handler::s_ctrl (int id, int value)
  * The used libv4l2 pixelformat is set to fmt, V4L2_FIELD_INTERLACED
  */
 void
-v4l2_handler::s_fmt (string fmtstr, __u32 xres, __u32 yres)
+v4l2_handler::s_fmt (std::string fmtstr, uint32_t xres, uint32_t yres)
 {
   if (streaming)
     {
@@ -663,6 +726,8 @@ v4l2_handler::s_fmt (string fmtstr, __u32 xres, __u32 yres)
       if (!fmtstr.empty())
         {
           fmt_code = v4l2_format_code(fmtstr.c_str());
+
+          //printf ("DEBUG: v4l2_format_code (%s) returned 0x%X = %s\n", fmtstr.c_str(), fmt_code, v4l2_fourcc_name (fmt_code).c_str());
           fmt.fmt.pix.pixelformat = fmt_code;
         }
       fmt.fmt.pix.field       = V4L2_FIELD_INTERLACED;
@@ -692,12 +757,12 @@ v4l2_handler::g_fmt ()
 
   octave_scalar_map ret;
   ret.assign ("size", s);
-  ret.assign ("pixelformat", std::string(v4l2_format_name(fmt.fmt.pix.pixelformat)));
+  ret.assign ("fourcc", std::string(v4l2_format_name(fmt.fmt.pix.pixelformat)));
   return ret;
 }
 
 /*!
- * https://www.kernel.org/doc/html/v4.9/media/uapi/v4l/vidioc-reqbufs.html
+ * https://www.kernel.org/doc/html/v6.1/userspace-api/media/vidioc-reqbufs.html
  * \param n number of buffers to initiate. A count value of zero frees all buffers.
  */
 void
@@ -760,11 +825,13 @@ v4l2_handler::qbuf ()
 
 /*!
  * \param nargout Number of output Parameter [image, sequence, timestamp, timecode]
- * \param preview 0=no preview, 1=show preview win if closed, 2=leave it closed
+ * \param preview
+ * \param raw_output return colorspace of image as raw as reasonable.
+ *        YUV formats are unpacked, semiplanar pages are merged but chroma subsampling and colorspace is left unchanged.
  * \return image, sequence, timestamp, [timecode]
  */
 octave_value_list
-v4l2_handler::capture (int nargout, int preview)
+v4l2_handler::capture (int nargout, bool preview, bool raw_output)
 {
   octave_value_list ret;
 
@@ -773,6 +840,9 @@ v4l2_handler::capture (int nargout, int preview)
       error("v4l2_handler::capture: Streaming wasn't enabled. Please use 'start(obj)'");
       return octave_value();
     }
+
+  //printf ("v4l2_handler::capture (%i, %i, %i)\n", nargout, preview, raw_output);
+
   struct v4l2_format fmt;
   CLEAR(fmt);
   fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
@@ -825,156 +895,101 @@ v4l2_handler::capture (int nargout, int preview)
   //octave_stdout << "INFO: width = " << fmt.fmt.pix.width << ", height = " << fmt.fmt.pix.height << endl;
   //octave_stdout << "INFO: Bytes captured = " << buf.bytesused << endl;
 
-  if ((fmt.fmt.pix.pixelformat != V4L2_PIX_FMT_RGB24) && preview)
-    {
-      error("v4l2_handler::capture: Preview is only available if VideoFormat is 'RGB3' aka 'RGB24' (V4L2_PIX_FMT_RGB24)");
-      preview = false;
-    }
+  // basic, underlying color space
+  bool is_rgb3  = false;
+  bool is_mjpg  = false;
+  bool is_ycbcr = false;
 
-  if (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_RGB24)
-    // RGB3 aka RGB24
+  if (   fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_RGB24
+      || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_BGR24)
+    // RGB3 aka RGB24 bzw. BGR
     // return [height x width x 3] uint8 matrix
     {
-      dim_vector dv (3, fmt.fmt.pix.width, fmt.fmt.pix.height);
-      uint8NDArray img (dv);
-      assert(img.numel() == int(buf.bytesused));
+      ret(0) = imaq_handler::get_RGB24 (buffers[buf.index].start,  buf.bytesused, fmt.fmt.pix.width, fmt.fmt.pix.height);
+      is_rgb3 = true;
 
-      unsigned char *p = reinterpret_cast<unsigned char*>(img.fortran_vec());
-      memcpy(p, buffers[buf.index].start, buf.bytesused);
-
-      Array<octave_idx_type> perm (dim_vector (3, 1));
-      perm(0) = 2;
-      perm(1) = 1;
-      perm(2) = 0;
-
-      ret(0) = octave_value(img.permute (perm));
+      if (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_BGR24)
+      {
+         // wtf? There must be a better way to do this...
+         // ret = ret (:,:,[3 2 1])
+         Array<octave::idx_vector> ia (dim_vector (3, 1), octave::idx_vector::colon);
+         Array<octave_idx_type> x (dim_vector (3, 1), 0);
+         x(0) = 2;
+         x(1) = 1;
+         x(2) = 0;
+         ia (2) = x;
+         ret(0) = ret(0).uint8_array_value().index (ia);
+      }
     }
   else if (  fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SBGGR10
-          || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGRBG10
-          || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SRGGB10
-          || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SBGGR12
-          || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGBRG12
-          || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGRBG12
-          || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SRGGB12 )
+             || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGRBG10
+             || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SRGGB10
+             || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SBGGR12
+             || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGBRG12
+             || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGRBG12
+             || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SRGGB12 )
     // RAW Bayer, 2 bytes per pixel
     // return [height * width] uint16 matrix
     {
-      dim_vector dv (fmt.fmt.pix.width, fmt.fmt.pix.height);
-      uint16NDArray img (dv);
-      assert(img.numel()*2 == int(buf.bytesused));
-      unsigned char *p = reinterpret_cast<unsigned char*>(img.fortran_vec());
-      memcpy(p, buffers[buf.index].start, buf.bytesused);
-      ret(0) = octave_value(img. transpose ());
+      ret(0) = imaq_handler::get_raw_bayer2 (buffers[buf.index].start, buf.bytesused, fmt.fmt.pix.width, fmt.fmt.pix.height);
     }
   else if ( fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SBGGR8
-         || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGBRG8
-         || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGRBG8
-         || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SRGGB8 )
+            || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGBRG8
+            || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SGRBG8
+            || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_SRGGB8 )
     // RAW Bayer, 1 byte per pixel
     // return [height * width] uint8 matrix
     {
-      dim_vector dv (fmt.fmt.pix.width, fmt.fmt.pix.height);
-      uint8NDArray img (dv);
-      assert(img.numel() == int(buf.bytesused));
-      unsigned char *p = reinterpret_cast<unsigned char*>(img.fortran_vec());
-      memcpy(p, buffers[buf.index].start, buf.bytesused);
-      ret(0) = octave_value(img.transpose ());
+      ret(0) = imaq_handler::get_raw_bayer1 (buffers[buf.index].start, buf.bytesused, fmt.fmt.pix.width, fmt.fmt.pix.height);
     }
   else if (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUYV)
     // YUYV aka YUV 4:2:2
-    // http://www.linuxtv.org/downloads/v4l-dvb-apis/V4L2-PIX-FMT-YUYV.html
+    // https://www.kernel.org/doc/html/v4.9/media/uapi/v4l/pixfmt-yuyv.html
+    // V4L2_PIX_FMT_YUYV is known in the Windows environment as YUY2
     // return struct with fields Y, Cb, Cr
     {
-      dim_vector dvy  (fmt.fmt.pix.width, fmt.fmt.pix.height);
-      dim_vector dvc (fmt.fmt.pix.width/2, fmt.fmt.pix.height);
-      uint8NDArray y (dvy);
-      uint8NDArray cb (dvc);
-      uint8NDArray cr (dvc);
-      assert ((y.numel() + cb.numel() + cr.numel()) == int(buf.bytesused));
-      unsigned int i;
-      unsigned char *s = reinterpret_cast<unsigned char*>(buffers[buf.index].start);
-      for (i=0; i < (fmt.fmt.pix.width * fmt.fmt.pix.height); ++i)
-        y(i) = s[2 * i];
-      for (i=0; i < (fmt.fmt.pix.width * fmt.fmt.pix.height / 2); ++i)
-        {
-          cb(i) = s[4 * i + 1];
-          cr(i) = s[4 * i + 3];
-        }
-
-      octave_scalar_map img;
-      img.assign ("Y", y.transpose ());
-      img.assign ("Cb", cb.transpose ());
-      img.assign ("Cr", cr.transpose ());
-      ret(0) = octave_value(img);
+      ret(0) = imaq_handler::get_YUYV (buffers[buf.index].start, buf.bytesused, fmt.fmt.pix.width, fmt.fmt.pix.height);
+      is_ycbcr = true;
     }
   else if (   fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YVU420
-           || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV420)
+              || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV420)
     // YVU420 aka YV12
-    // http://www.linuxtv.org/downloads/v4l-dvb-apis/re23.html
+    // https://www.kernel.org/doc/html/v4.9/media/uapi/v4l/pixfmt-yuv420.html
     {
-      dim_vector dvy  (fmt.fmt.pix.width, fmt.fmt.pix.height);
-      dim_vector dvc (fmt.fmt.pix.width/2, fmt.fmt.pix.height/2);
-      uint8NDArray y (dvy);
-      uint8NDArray c1 (dvc);
-      uint8NDArray c2 (dvc);
-      assert ((y.numel() + c1.numel() + c2.numel()) == int(buf.bytesused));
-
-      // Y
-      unsigned char *p = reinterpret_cast<unsigned char*>(y.fortran_vec());
-      memcpy(p, buffers[buf.index].start, y.numel ());
-
-      // C1
-      p = reinterpret_cast<unsigned char*>(c1.fortran_vec());
-      memcpy(p, (unsigned char*)buffers[buf.index].start + y.numel (), c1.numel ());
-
-      // C2
-      p = reinterpret_cast<unsigned char*>(c2.fortran_vec());
-      memcpy(p, (unsigned char*)buffers[buf.index].start + y.numel () + c1.numel (), c2.numel ());
-
-      octave_scalar_map img;
-      img.assign ("Y", y.transpose ());
-      if (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV420)
-        {
-          img.assign ("Cb", c1.transpose ());
-          img.assign ("Cr", c2.transpose ());
-        }
-      else
-        // V4L2_PIX_FMT_YVU420
-        {
-          img.assign ("Cb", c2.transpose ());
-          img.assign ("Cr", c1.transpose ());
-        }
-
-      ret(0) = octave_value(img);
+      ret(0) = imaq_handler::get_YVU420 (buffers[buf.index].start, buf.bytesused, fmt.fmt.pix.width, fmt.fmt.pix.height, fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_YUV420);
+      is_ycbcr = true;
+    }
+  else if (   fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_NV12
+              || fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_NV21)
+    // https://www.kernel.org/doc/html/v4.9/media/uapi/v4l/pixfmt-nv12.html
+    {
+      ret(0) = imaq_handler::get_NV12 (buffers[buf.index].start, buf.bytesused, fmt.fmt.pix.width, fmt.fmt.pix.height);
+      is_ycbcr = true;
     }
   else
-    // No conversion for this format
-    // http://www.linuxtv.org/downloads/v4l-dvb-apis/ch02s10.html
-    // Just return the bytes as vector in this case.
+    // No conversion for this format (perhaps a compressed format like MPEG)
+    // https://www.kernel.org/doc/html/v6.1/userspace-api/media/v4l/pixfmt-compressed.html
+    // -> Just return the bytes as vector in this case.
     {
       //octave_stdout << "INFO: No conversion for "
       //              << v4l2_format_name(fmt.fmt.pix.pixelformat)
       //              << " implemented, returning raw stream..." << endl;
-
-      dim_vector dv (buf.bytesused, 1);
-      uint8NDArray img (dv);
-      unsigned char *p = reinterpret_cast<unsigned char*>(img.fortran_vec());
-      memcpy(p, buffers[buf.index].start, buf.bytesused);
-      ret(0) = octave_value(img);
+      ret(0) = imaq_handler::get_raw_bytes (buffers[buf.index].start, buf.bytesused);
+      is_mjpg = (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_MPEG) || (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_MJPEG);
     }
 
+  // return sequence
   if (nargout > 1)
     ret(1) = octave_value(buf.sequence);
 
+  // return timestamp. Attention, there is also an calculated timestamp above
   if (nargout > 2)
     {
       // add timestamp to frame
-      octave_scalar_map timestamp;
-      timestamp.assign ("tv_sec", (long int)(buf.timestamp.tv_sec));
-      timestamp.assign ("tv_usec", (long int)(buf.timestamp.tv_usec));
-      ret(2) = octave_value(timestamp);
+      ret(2) = octave_value(buf.timestamp.tv_sec + buf.timestamp.tv_usec / 1e6);
     }
 
+  // return timecode
   if (nargout > 3)
     {
       if (buf.flags & V4L2_BUF_FLAG_TIMECODE)
@@ -995,42 +1010,61 @@ v4l2_handler::capture (int nargout, int preview)
         }
     }
 
-  // use preview window?
-  if (preview)
+  // Do we need an rgb image?
+  if (! raw_output || preview)
+  {
+    octave_value rgb_img;
+    //printf ("DEBUG: we need a RGB3 image...\n");
+    if (!is_rgb3)
     {
-      if (!preview_window)
-        {
-          preview_window = new img_win(10, 10, fmt.fmt.pix.width, fmt.fmt.pix.height);
-          preview_window->show();
-        }
-      if (preview_window)
-        {
-          if(preview == 1 && !preview_window->shown())
-            preview_window->show();
+      //printf ("DEBUG: we need to convert %s to RGB3...\n", v4l2_format_name(fmt.fmt.pix.pixelformat).c_str());
 
-          // We can only use preview for RGB24 aka RGB3
-          if (fmt.fmt.pix.pixelformat == V4L2_PIX_FMT_RGB24)
-            {
-              // octave_stdout << "Bytes captured = " << buf.bytesused << endl;
-              // sanity checks
-              if (buf.bytesused != (3 *  fmt.fmt.pix.width * fmt.fmt.pix.height))
-                error ("v4l2_handler::capture: Returned size of buffer doesn't match 3 * width * height");
-              else
-                {
-                  preview_window->copy_img(reinterpret_cast<unsigned char*>(buffers[buf.index].start),
-                                           fmt.fmt.pix.width, fmt.fmt.pix.height, 1);
-                  preview_window->custom_label(dev.c_str(), buf.sequence, 1.0/dt);
-                }
-            }
-        }
-    }
-  else if (preview_window)
-    {
-      delete preview_window;
-      preview_window = 0;
+      if (is_ycbcr)
+        rgb_img = YCbCr_to_RGB (ret(0), ITU_standard);
+      else if (is_mjpg)
+        rgb_img = JPG_to_RGB (ret(0));
+      else
+        error ("v4l2_handler::capture: no conversion from '%s' to RGB3 implemented yet", v4l2_format_name(fmt.fmt.pix.pixelformat).c_str());
     }
+    //else
+      //printf ("DEBUG: already an RGB3 image, do nothing...\n");
+
+    if (preview)
+      {
+        if (!preview_window)
+          {
+            preview_window = new img_win(10, 10, fmt.fmt.pix.width, fmt.fmt.pix.height);
+            preview_window->show();
+          }
+        if (preview_window)
+          {
+            if(preview == 1 && !preview_window->shown())
+              preview_window->show();
+
+            uint8NDArray tmp = rgb_img.uint8_array_value();
+            Array<octave_idx_type> perm (dim_vector (3, 1));
+            perm(0) = 2;
+            perm(1) = 1;
+            perm(2) = 0;
+            tmp = tmp.permute (perm);
+            unsigned char *p = reinterpret_cast<unsigned char*>(tmp.fortran_vec());
+
+            preview_window->copy_img(p, fmt.fmt.pix.width, fmt.fmt.pix.height, 1);
+            preview_window->custom_label(dev.c_str(), buf.sequence, 1.0/dt);
+          }
+      }
+    else if (preview_window)
+      {
+        delete preview_window;
+        preview_window = 0;
+      }
+
+    if (! raw_output && ! is_rgb3)
+      ret(0) = rgb_img;
+  }
 
   xioctl(fd, VIDIOC_QBUF, &buf);
+
   return ret;
 }
 
@@ -1040,7 +1074,8 @@ v4l2_handler::capture (int nargout, int preview)
 void
 v4l2_handler::capture_to_ppm (const char *fn)
 {
-  uint8NDArray img = capture (1, 0)(0).uint8_array_value();
+  NDArray rgb = capture (1, 0, 1)(0).array_value();
+  uint8NDArray img = rgb * 255.0;
   Matrix per(3,1);
   per(0) = 2;
   per(1) = 1;
@@ -1137,17 +1172,19 @@ v4l2_handler::close ()
   fd = -1;
 }
 
-v4l2_handler*
-get_v4l2_handler_from_ov (octave_value ov)
-{
-  if (ov.type_id() != v4l2_handler::static_type_id())
-    {
-      error("get_v4l2_handler_from_ov: Not a valid v4l2_handler");
-      return 0;
-    }
+//~ v4l2_handler*
+//~ get_v4l2_handler_from_ov (octave_value ov)
+//~ {
+//~ if (ov.type_id() != v4l2_handler::static_type_id())
+//~ {
+//~ error("get_v4l2_handler_from_ov: Not a valid v4l2_handler");
+//~ return 0;
+//~ }
 
-  v4l2_handler* imgh = 0;
-  const octave_base_value& rep = ov.get_rep();
-  imgh = &((v4l2_handler &)rep);
-  return imgh;
-}
+//~ v4l2_handler* imgh = 0;
+//~ const octave_base_dld_value& rep = ov.get_rep();
+//~ imgh = &((v4l2_handler &)rep);
+//~ return imgh;
+//~ }
+
+#endif
