@@ -1,4 +1,4 @@
-// Copyright (C) 2014 Andreas Weber <andy.weber.aw@gmail.com>
+// Copyright (C) 2014-2025 Andreas Weber <andy.weber.aw@gmail.com>
 //
 // This program is free software; you can redistribute it and/or modify it under
 // the terms of the GNU General Public License as published by the Free Software
@@ -33,10 +33,9 @@
 
 #include <octave/dMatrix.h>
 #include <iostream>
+#include "cl_imaq_handler.h"
 #include "cl_img_win.h"
 
-using namespace std;
-
 #define CLEAR(x) memset(&(x), 0, sizeof(x))
 #define xioctl(n, r, p) xioctl_name(n, r, p, #r, __FILE__, __LINE__)
 
@@ -51,51 +50,56 @@ struct buffer
  * v4l2 wrapper for octave-image-acquisition
  *
  * A big help was the Video Grabber example using libv4l by Mauro Carvalho Cheha
- * http://www.linuxtv.org/downloads/v4l-dvb-apis/v4l2grab-example.html
+ * https://www.linuxtv.org/downloads/v4l-dvb-apis/userspace-api/v4l/v4l2grab-example.html
  */
-class v4l2_handler: public octave_base_value
+class v4l2_handler: public imaq_handler
 {
 public:
 
   v4l2_handler ();
 
-  octave_base_value *clone (void) const // TODO: check if this is okay
+  octave_base_dld_value *clone (void) const // TODO: check if this is okay
   {
-    octave_stdout << "v4l2_handler clone" << endl;
+    octave_stdout << "v4l2_handler clone" << std::endl;
     return new v4l2_handler (*this);
   }
 
-  octave_base_value *empty_clone (void) const // TODO: check if this is okay
+  octave_base_dld_value *empty_clone (void) const // TODO: check if this is okay
   {
-    octave_stdout << "v4l2_handler empty_clone" << endl;
+    octave_stdout << "v4l2_handler empty_clone" << std::endl;
     return new v4l2_handler ();
   }
 
   ~v4l2_handler (void);
 
-  octave_scalar_map open (string d, bool quiet); //!< open a v4l2 device e.g. /dev/video0
-  void print (std::ostream& os, bool pr_as_read_syntax) const;  //!< print itself on ostream
-  octave_value querycap ();        //!< Query device capabilities
+  octave_map enum_devices ();
 
-  octave_value enuminput ();       //!< Enumerate video inputs
-  int g_input ();                  //!< Query the current video input
-  void s_input (int index);        //!< Select video input
+  octave_scalar_map open (std::string d, bool quiet); //!< open a v4l2 device e.g. /dev/video0
+  void print (std::ostream& os, bool pr_as_read_syntax);  //!< print itself on ostream
+  octave_value querycap ();          //!< Query device capabilities
 
-  octave_value enum_fmt (enum v4l2_buf_type type = V4L2_BUF_TYPE_VIDEO_CAPTURE); //!< Enumerate image formats
-  Matrix enum_framesizes (string pixelformat);     //!< Enumerate frame sizes
+  octave_value enum_inputs ();       //!< Enumerate video inputs
+  int get_input ();                  //!< Query the current video input
+  void set_input (int index);        //!< Select video input
+
+  octave_value enum_formats ();      //!< Enumerate image formats
+
+  Matrix enum_framesizes (std::string pixelformat);     //!< Enumerate frame sizes
   octave_scalar_map g_fmt ();                      //!< Get current format
-  void s_fmt (string fmtstr, __u32 xres, __u32 yres); //!< Set format
+  void s_fmt (std::string fmtstr, uint32_t xres, uint32_t yres); //!< Set format
 
-  Matrix enum_frameintervals (string pixelformat, __u32 width, __u32 height);     //!< Enumerate frame intervals
-  Matrix g_parm ();                    //!< Get streaming parameters (like frame interval)
-  void s_parm (Matrix timeperframe);   //!< Set streaming parameters (like frame interval)
+  Matrix enum_frameintervals (std::string pixelformat, uint32_t width, uint32_t height);     //!< Enumerate frame intervals
+  Matrix get_frameinterval ();
+  void set_frameinterval (Matrix timeperframe);
 
   octave_value queryctrl ();                  //!< Query controls
   int g_ctrl (int id);                        //!< Get control
-  void s_ctrl (int id, int value);            //!< Set control
+  void s_ctrl (int id, octave_value val);     //!< Set control
 
   octave_value_list capture (int nargout,
-                             int preview=0);  //!< Retrieve captured image from buffer
+                             bool preview,
+                             bool raw_output);//!< Retrieve captured image from buffer
+
   void capture_to_ppm (const char *fn);       //!< Retrieve captured image from buffer and save it as ppm
 
   void streamon (unsigned int n);             //!< start streaming with n buffers
@@ -103,38 +107,31 @@ public:
 
   void close ();                              //!< close v4l2 device
 
-  bool preview_window_is_shown()
+  bool is_video_capture ()
   {
-    Fl::wait(0);
-    return (preview_window)? preview_window->shown() : false;
+    return _is_video_capture;
   }
 
-  bool is_video_capture () {return _is_video_capture;}
-  bool is_meta_capture () {return _is_meta_capture;}
+  // see https://unix.stackexchange.com/questions/512759/multiple-dev-video-for-one-physical-device
+  // why today video0 and video1 are created simulateneously
+  bool is_meta_capture ()
+  {
+    return _is_meta_capture;
+  }
 
 private:
   v4l2_handler (const v4l2_handler& m);
   static bool type_loaded;
 
   int fd;
-  string dev;
+  std::string dev;
   unsigned int n_buffer;
   struct buffer *buffers;
   bool streaming;
-  img_win *preview_window;
+  //img_win *preview_window;
   bool _is_video_capture;
   bool _is_meta_capture;
 
-  // Properties
-  bool is_constant (void) const
-  {
-    return true;
-  }
-  bool is_defined (void) const
-  {
-    return true;
-  }
-
   void xioctl_name (int fh, unsigned long int request, void *arg, const char* name, const char* file, const int line);
   octave_scalar_map get_osm (struct v4l2_queryctrl queryctrl);
   void reqbufs (unsigned int n);  //!< Initiate Memory Mapping or User Pointer I/O
@@ -142,10 +139,6 @@ private:
   void qbuf ();
   void munmap ();
   octave_scalar_map expand_cap (unsigned int cap);
-
-  DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
 };
 
-v4l2_handler* get_v4l2_handler_from_ov (octave_value ov);
-
 #endif
